@@ -1,22 +1,22 @@
 import os
 import uuid
 import logging
+import collections
 from functools import lru_cache
 from copy import copy, deepcopy
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 from typing import Mapping, Tuple, Type
 from linkml_runtime.utils.namespaces import Namespaces
 from deprecated.classic import deprecated
 from linkml_runtime.utils.context_utils import parse_import_map
 from linkml_runtime.linkml_model.meta import *
-
+from enum import Enum
 logger = logging.getLogger(__name__)
 
 
 MAPPING_TYPE = str  ## e.g. broad, exact, related, ...
 CACHE_SIZE = 1024
 
-
 SLOTS = 'slots'
 CLASSES = 'classes'
 ENUMS = 'enums'
@@ -30,7 +30,13 @@ TYPE_NAME = Union[TypeDefinitionName, str]
 ENUM_NAME = Union[EnumDefinitionName, str]
 
 
-def _closure(f, x, reflexive=True, **kwargs):
+class OrderedBy(Enum):
+    RANK = "rank"
+    LEXICAL = "lexical"
+    PRESERVE = "preserve"
+
+
+def _closure(f, x, reflexive=True, depth_first=True, **kwargs):
     if reflexive:
         rv = [x]
     else:
@@ -38,13 +44,18 @@ def _closure(f, x, reflexive=True, **kwargs):
     visited = []
     todo = [x]
     while len(todo) > 0:
-        i = todo.pop()
+        if depth_first:
+            i = todo.pop()
+        else:
+            i = todo[0]
+            todo = todo[1:]
         visited.append(i)
         vals = f(i)
         for v in vals:
             if v not in visited:
                 todo.append(v)
-                rv.append(v)
+                if v not in rv:
+                    rv.append(v)
     return rv
 
 
@@ -193,13 +204,60 @@ class SchemaView(object):
         """
         return self._get_dict(CLASSES, imports)
 
-    @lru_cache()
-    def all_classes(self, imports=True) -> Dict[ClassDefinitionName, ClassDefinition]:
+    def _order_lexically(self, elements: dict):
         """
+        :param element: slots or class type to order
+        :param imports
+        :return: all classes or slots sorted lexically in schema view
+        """
+        ordered_list_of_names = []
+        ordered_elements = {}
+        for c in elements:
+            ordered_list_of_names.append(c)
+        ordered_list_of_names.sort()
+        for name in ordered_list_of_names:
+            ordered_elements[self.get_element(name).name] = self.get_element(name)
+        return ordered_elements
+
+    def _order_rank(self, elements: dict):
+        """
+        :param elements: slots or classes to order
+        :return: all classes or slots sorted by their rank in schema view
+        """
+
+        rank_map = {}
+        unranked_map = {}
+        rank_ordered_elements = {}
+        for name, definition in elements.items():
+            if definition.rank is None:
+                unranked_map[self.get_element(name).name] = self.get_element(name)
+
+            else:
+                rank_map[definition.rank] = name
+        rank_ordered_map = collections.OrderedDict(sorted(rank_map.items()))
+        for k, v in rank_ordered_map.items():
+            rank_ordered_elements[self.get_element(v).name] = self.get_element(v)
+
+        rank_ordered_elements.update(unranked_map)
+        return rank_ordered_elements
+
+    @lru_cache()
+    def all_classes(self, ordered_by=OrderedBy.PRESERVE, imports=True) -> Dict[ClassDefinitionName, ClassDefinition]:
+        """
+        :param ordered_by: an enumerated parameter that returns all the slots in the order specified.
         :param imports: include imports closure
         :return: all classes in schema view
         """
-        return self._get_dict(CLASSES, imports)
+        classes = copy(self._get_dict(CLASSES, imports))
+
+        if ordered_by == OrderedBy.LEXICAL:
+            ordered_classes = self._order_lexically(elements=classes)
+        elif ordered_by == OrderedBy.RANK:
+            ordered_classes = self._order_rank(elements=classes)
+        else:  # else preserve the order in the yaml
+            ordered_classes = classes
+
+        return ordered_classes
 
     @deprecated("Use `all_slots` instead")
     @lru_cache()
@@ -211,19 +269,29 @@ class SchemaView(object):
         return self.all_slots(**kwargs)
 
     @lru_cache()
-    def all_slots(self, imports=True, attributes=True) -> Dict[SlotDefinitionName, SlotDefinition]:
+    def all_slots(self, ordered_by=OrderedBy.PRESERVE, imports=True, attributes=True) -> Dict[SlotDefinitionName, SlotDefinition]:
         """
+        :param ordered_by: an enumerated parameter that returns all the slots in the order specified.
         :param imports: include imports closure
+        :param attributes: include attributes as slots or not, default is to include.
         :return: all slots in schema view
         """
+
         slots = copy(self._get_dict(SLOTS, imports))
         if attributes:
             for c in self.all_classes().values():
                 for aname, a in c.attributes.items():
                     if aname not in slots:
                         slots[aname] = a
-        return slots
 
+        if ordered_by == OrderedBy.LEXICAL:
+            ordered_slots = self._order_lexically(elements=slots)
+        elif ordered_by == OrderedBy.RANK:
+            ordered_slots = self._order_rank(elements=slots)
+        else:
+            # preserve order in YAML
+            ordered_slots = slots
+        return ordered_slots
 
     @deprecated("Use `all_enums` instead")
     @lru_cache()
@@ -283,11 +351,11 @@ class SchemaView(object):
         :param imports: include imports closure
         :return: all elements in schema view
         """
-        all_classes = self.all_classes(imports)
-        all_slots = self.all_slots(imports)
-        all_enums = self.all_enums(imports)
-        all_types = self.all_types(imports)
-        all_subsets = self.all_subsets(imports)
+        all_classes = self.all_classes(imports=imports)
+        all_slots = self.all_slots(imports=imports)
+        all_enums = self.all_enums(imports=imports)
+        all_types = self.all_types(imports=imports)
+        all_subsets = self.all_subsets(imports=imports)
         # {**a,**b} syntax merges dictionary a and b into a single dictionary, removing duplicates.
         return {**all_classes, **all_slots, **all_enums, **all_types, **all_subsets}
 
@@ -297,11 +365,11 @@ class SchemaView(object):
         :param imports: include imports closure
         :return: all elements in schema view
         """
-        all_classes = self.all_classes(imports)
-        all_slots = self.all_slots(imports)
-        all_enums = self.all_enums(imports)
-        all_types = self.all_types(imports)
-        all_subsets = self.all_subsets(imports)
+        all_classes = self.all_classes(imports=imports)
+        all_slots = self.all_slots(imports=imports)
+        all_enums = self.all_enums(imports=imports)
+        all_types = self.all_types(imports=imports)
+        all_subsets = self.all_subsets(imports=imports)
         # {**a,**b} syntax merges dictionary a and b into a single dictionary, removing duplicates.
         return {**all_classes, **all_slots, **all_enums, **all_types, **all_subsets}
 
@@ -312,6 +380,7 @@ class SchemaView(object):
         for s in schemas:
             # get the value of element name from the schema, if empty, return empty dictionary.
             d1 = getattr(s, slot_name, {})
+            # {**d,**d1} syntax merges dictionary a and b into a single dictionary, removing duplicates.
             d = {**d, **d1}
 
         return d
@@ -376,7 +445,7 @@ class SchemaView(object):
         :param imports: include import closure
         :return: class definition
         """
-        c = self.all_classes(imports).get(class_name, None)
+        c = self.all_classes(imports=imports).get(class_name, None)
         if strict and c is None:
             raise ValueError(f'No such class as "{class_name}"')
         else:
@@ -389,9 +458,9 @@ class SchemaView(object):
         :param imports: include import closure
         :return: slot definition
         """
-        slot = self.all_slots(imports).get(slot_name, None)
+        slot = self.all_slots(imports=imports).get(slot_name, None)
         if slot is None and attributes:
-            for c in self.all_classes(imports).values():
+            for c in self.all_classes(imports=imports).values():
                 if slot_name in c.attributes:
                     if slot is not None:
                         # slot name is ambiguous, no results
@@ -482,7 +551,7 @@ class SchemaView(object):
         :param is_a: include is_a parents (default is True)
         :return: all direct child class names (is_a and mixins)
         """
-        elts = [self.get_class(x) for x in self.all_classes(imports)]
+        elts = [self.get_class(x) for x in self.all_classes(imports=imports)]
         return [x.name for x in elts if (x.is_a == class_name and is_a) or (mixins and class_name in x.mixins)]
 
     @lru_cache()
@@ -498,7 +567,8 @@ class SchemaView(object):
         return [x.name for x in elts if (x.is_a == slot_name and is_a) or (mixins and slot_name in x.mixins)]
 
     @lru_cache()
-    def class_ancestors(self, class_name: CLASS_NAME, imports=True, mixins=True, reflexive=True, is_a=True) -> List[ClassDefinitionName]:
+    def class_ancestors(self, class_name: CLASS_NAME, imports=True, mixins=True, reflexive=True, is_a=True,
+                        depth_first=True) -> List[ClassDefinitionName]:
         """
         Closure of class_parents method
 
@@ -507,9 +577,12 @@ class SchemaView(object):
         :param mixins: include mixins (default is True)
         :param is_a: include is_a parents (default is True)
         :param reflexive: include self in set of ancestors
+        :param depth_first:
         :return: ancestor class names
         """
-        return _closure(lambda x: self.class_parents(x, imports=imports, mixins=mixins, is_a=is_a), class_name, reflexive=reflexive)
+        return _closure(lambda x: self.class_parents(x, imports=imports, mixins=mixins, is_a=is_a),
+                        class_name,
+                        reflexive=reflexive,  depth_first=depth_first)
 
     @lru_cache()
     def slot_ancestors(self, slot_name: SLOT_NAME, imports=True, mixins=True, reflexive=True, is_a=True) -> List[SlotDefinitionName]:
@@ -523,7 +596,9 @@ class SchemaView(object):
         :param reflexive: include self in set of ancestors
         :return: ancestor slot names
         """
-        return _closure(lambda x: self.slot_parents(x, imports=imports, mixins=mixins, is_a=is_a), slot_name, reflexive=reflexive)
+        return _closure(lambda x: self.slot_parents(x, imports=imports, mixins=mixins, is_a=is_a),
+                        slot_name,
+                        reflexive=reflexive)
 
     @lru_cache()
     def class_descendants(self, class_name: CLASS_NAME, imports=True, mixins=True, reflexive=True, is_a=True) -> List[ClassDefinitionName]:
